# coding: utf-8

# flake8: noqa
"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from optimeering_beta.models.access_key_created import AccessKeyCreated
from optimeering_beta.models.access_key_post_response import AccessKeyPostResponse
from optimeering_beta.models.access_post_key import AccessPostKey
from optimeering_beta.models.end import End
from optimeering_beta.models.enum_parameters import EnumParameters
from optimeering_beta.models.http_validation_error import HTTPValidationError
from optimeering_beta.models.location_inner import LocationInner
from optimeering_beta.models.max_event_time import MaxEventTime
from optimeering_beta.models.prediction_float_model import PredictionFloatModel
from optimeering_beta.models.predictions_created_series import PredictionsCreatedSeries
from optimeering_beta.models.predictions_data_get_response import PredictionsDataGetResponse
from optimeering_beta.models.predictions_dict_model import PredictionsDictModel
from optimeering_beta.models.predictions_inner import PredictionsInner
from optimeering_beta.models.predictions_series_get_response import PredictionsSeriesGetResponse
from optimeering_beta.models.predictions_single_event_data_created import PredictionsSingleEventDataCreated
from optimeering_beta.models.predictions_single_series_data_created import PredictionsSingleSeriesDataCreated
from optimeering_beta.models.start import Start
from optimeering_beta.models.validation_error import ValidationError

# add to __all__
__all__ = [
    "AccessKeyCreated",
    "AccessKeyPostResponse",
    "AccessPostKey",
    "End",
    "EnumParameters",
    "HTTPValidationError",
    "LocationInner",
    "MaxEventTime",
    "PredictionFloatModel",
    "PredictionsCreatedSeries",
    "PredictionsDataGetResponse",
    "PredictionsDictModel",
    "PredictionsInner",
    "PredictionsSeriesGetResponse",
    "PredictionsSingleEventDataCreated",
    "PredictionsSingleSeriesDataCreated",
    "Start",
    "ValidationError",
]



# coding: utf-8

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from typing import Dict, List, Optional, Tuple, Union

from optimeering_beta.api_client import OptimeeringClient, RequestSerialized
from optimeering_beta.models.enum_parameters import EnumParameters
from pydantic import Field, StrictFloat, validate_call
from typing_extensions import Annotated


class ParametersApi:
    """
    Collection of methods to interact with ParametersApi
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = OptimeeringClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_parameter_values(
        self,
        param: EnumParameters,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> object:
        """Parameter List


        :param param: (required)
        :type param: EnumParameters
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        :rtype: object

        :Example:

        >>> from optimeering_beta import Configuration, OptimeeringClient
        >>> configuration = Configuration(host="https://beta.optimeering.com")
        >>> client = OptimeeringClient(configuration=configuration)
        >>> # Post data point - replace ... with correct dataformat documented above
        >>> response = client.parameters_api.get_parameter_values(...)

        """  # noqa: E501

        _param = self._get_parameter_values_serialize(
            param=param,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "object",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        response = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data
        if type(response) != dict:
            response._client = self.api_client
        return response

    def _get_parameter_values_serialize(
        self,
        param,
    ) -> RequestSerialized:
        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if param is not None:
            _path_params["param"] = param.value
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        if self.api_client.configuration.api_key is None:
            # inject azure token
            _header_params["Authorization"] = f"Bearer {self.api_client.token}"
        else:
            _header_params["apikey"] = self.api_client.configuration.api_key

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/parameters/{param}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
        )



# coding: utf-8

# flake8: noqa

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "0.0.4"

# import apis into sdk package
from optimeering_beta.api.access_api import AccessApi
from optimeering_beta.api.parameters_api import ParametersApi
from optimeering_beta.api.predictions_api import PredictionsApi
from optimeering_beta.api_client import OptimeeringClient

# import OptimeeringClient
from optimeering_beta.api_response import ApiResponse
from optimeering_beta.azure_authentication import AzureAuth
from optimeering_beta.configuration import Configuration
from optimeering_beta.exceptions import (
    ApiAttributeError,
    ApiException,
    ApiKeyError,
    ApiTypeError,
    ApiValueError,
    OpenApiException,
)

# import models into sdk package
from optimeering_beta.models.access_key_created import AccessKeyCreated
from optimeering_beta.models.access_key_post_response import AccessKeyPostResponse
from optimeering_beta.models.access_post_key import AccessPostKey
from optimeering_beta.models.end import End
from optimeering_beta.models.enum_parameters import EnumParameters
from optimeering_beta.models.http_validation_error import HTTPValidationError
from optimeering_beta.models.location_inner import LocationInner
from optimeering_beta.models.max_event_time import MaxEventTime
from optimeering_beta.models.prediction_float_model import PredictionFloatModel
from optimeering_beta.models.predictions_created_series import PredictionsCreatedSeries
from optimeering_beta.models.predictions_data_get_response import PredictionsDataGetResponse
from optimeering_beta.models.predictions_dict_model import PredictionsDictModel
from optimeering_beta.models.predictions_inner import PredictionsInner
from optimeering_beta.models.predictions_series_get_response import PredictionsSeriesGetResponse
from optimeering_beta.models.predictions_single_event_data_created import PredictionsSingleEventDataCreated
from optimeering_beta.models.predictions_single_series_data_created import PredictionsSingleSeriesDataCreated
from optimeering_beta.models.start import Start
from optimeering_beta.models.validation_error import ValidationError

# add to __all__
__all__ = [
    "AccessApi",
    "ParametersApi",
    "PredictionsApi",
    "ApiResponse",
    "AzureAuth",
    "OptimeeringClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "AccessKeyCreated",
    "AccessKeyPostResponse",
    "AccessPostKey",
    "End",
    "EnumParameters",
    "HTTPValidationError",
    "LocationInner",
    "MaxEventTime",
    "PredictionFloatModel",
    "PredictionsCreatedSeries",
    "PredictionsDataGetResponse",
    "PredictionsDictModel",
    "PredictionsInner",
    "PredictionsSeriesGetResponse",
    "PredictionsSingleEventDataCreated",
    "PredictionsSingleSeriesDataCreated",
    "Start",
    "ValidationError",
]



# coding: utf-8

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Dict, List, Optional, Tuple, Union

from optimeering_beta.api_client import OptimeeringClient, RequestSerialized
from optimeering_beta.models.predictions_data_get_response import PredictionsDataGetResponse
from optimeering_beta.models.predictions_series_get_response import PredictionsSeriesGetResponse
from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated


class PredictionsApi:
    """
    Collection of methods to interact with PredictionsApi
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = OptimeeringClient.get_default()
        self.api_client = api_client

    @validate_call
    def get_latest_predictions(
        self,
        max_event_time: Annotated[
            Optional[Any],
            Field(
                description="If specified, will only return the latest prediction available at the specified time. If not specified, no filters are applied. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)"
            ),
        ] = None,
        series_id: Annotated[
            Optional[List[StrictInt]],
            Field(description="Series ID to filter. If not specified, will return all series."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> PredictionsDataGetResponse:
        """Get Latest Predictions

        Returns latest predictions

        :param max_event_time: If specified, will only return the latest prediction available at the specified time. If not specified, no filters are applied. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)
        :type max_event_time: MaxEventTime
        :param series_id: Series ID to filter. If not specified, will return all series.
        :type series_id: List[StrictInt]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        :rtype: PredictionsDataGetResponse

        :Example:

        >>> from optimeering_beta import Configuration, OptimeeringClient
        >>> configuration = Configuration(host="https://beta.optimeering.com")
        >>> client = OptimeeringClient(configuration=configuration)
        >>> # Get filtered data point - replace ... with appropriate filters documented above
        >>> response = client.predictions_api.get_latest_predictions(...)

        """  # noqa: E501

        _param = self._get_latest_predictions_serialize(
            max_event_time=max_event_time,
            series_id=series_id,
            limit=None,
            offset=None,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PredictionsDataGetResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        paginated_response = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data
        if type(paginated_response) != dict:
            paginated_response._client = self.api_client
        next_page = paginated_response.next_page

        if next_page is None:
            return paginated_response

        # resolve extendable attribute
        attribute_to_extend_selected: Optional[str] = None
        if "series_id" in set(paginated_response.items[0].model_fields):
            extendable_attribute = {"datapoints", "events"}
            attribute_to_extend = extendable_attribute.intersection(set(paginated_response.items[0].model_fields))
            if len(attribute_to_extend) == 1:
                attribute_to_extend_selected = list(attribute_to_extend)[0]
            else:
                raise AttributeError("Failed to resolve extendable attribute.")

        api_key_configured = self.api_client.configuration.api_key is not None
        if api_key_configured:
            auth_header = {"apikey": self.api_client.configuration.api_key}
        while next_page is not None:
            if not api_key_configured:
                auth_header = {"Authorization": f"Bearer {self.api_client.token}"}

            next_pagination = self.api_client.call_api(
                method=_param[0],
                url=next_page,
                header_params={**_param[2], **auth_header},
                body=_param[3],
                post_params=_param[4],
                _request_timeout=_request_timeout,
            )
            next_pagination.read()
            next_pagination = self.api_client.response_deserialize(
                response_data=next_pagination,
                response_types_map=_response_types_map,
            ).data
            next_page = next_pagination.next_page

            if attribute_to_extend_selected:
                if next_pagination.items[0].series_id == paginated_response.items[-1].series_id:
                    # if series id is same, extract the first item and extend to last item
                    continued_data: List = getattr(next_pagination.items.pop(0), attribute_to_extend_selected)
                    previous_data: List = getattr(paginated_response.items[-1], attribute_to_extend_selected)
                    previous_data.extend(continued_data)
            paginated_response.items.extend(next_pagination.items)
        return paginated_response

    def _get_latest_predictions_serialize(
        self,
        max_event_time,
        series_id,
        limit,
        offset,
    ) -> RequestSerialized:
        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if max_event_time is not None:
            _query_params.append(("max_event_time", max_event_time))

        if series_id is not None:
            series_id = ",".join(str(i) for i in series_id)
            _query_params.append(("series_id", series_id))

        if limit is not None:
            _query_params.append(("limit", limit))

        if offset is not None:
            _query_params.append(("offset", offset))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        if self.api_client.configuration.api_key is None:
            # inject azure token
            _header_params["Authorization"] = f"Bearer {self.api_client.token}"
        else:
            _header_params["apikey"] = self.api_client.configuration.api_key

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/predictions/latest",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
        )

    @validate_call
    def get_prediction_series(
        self,
        id: Annotated[
            Optional[List[StrictInt]],
            Field(description="ID of the series to retrieve. If not specified, will return all series."),
        ] = None,
        product: Annotated[
            Optional[List[StrictStr]],
            Field(
                description="The product for which series should be retrieved. If not specified, will return series for all products."
            ),
        ] = None,
        unit_type: Annotated[
            Optional[List[StrictStr]],
            Field(
                description="Unit type (eq. 'volumes', 'price', or 'price_spread'). If not specified, will return series for all unit types."
            ),
        ] = None,
        statistic: Annotated[
            Optional[List[StrictStr]],
            Field(
                description="Statistic type (eg. 'size', 'direction', or 'quantile'). If not specified, will return series for all statistic types."
            ),
        ] = None,
        area: Annotated[
            Optional[List[StrictStr]],
            Field(description="The name of the area (eg - NO3, SE4, or FI). If not specified, will return all areas."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> PredictionsSeriesGetResponse:
        """Get Series

        Returns the prediction series

        :param id: ID of the series to retrieve. If not specified, will return all series.
        :type id: List[StrictInt]
        :param product: The product for which series should be retrieved. If not specified, will return series for all products.
        :type product: List[StrictStr]
        :param unit_type: Unit type (eq. 'volumes', 'price', or 'price_spread'). If not specified, will return series for all unit types.
        :type unit_type: List[StrictStr]
        :param statistic: Statistic type (eg. 'size', 'direction', or 'quantile'). If not specified, will return series for all statistic types.
        :type statistic: List[StrictStr]
        :param area: The name of the area (eg - NO3, SE4, or FI). If not specified, will return all areas.
        :type area: List[StrictStr]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        :rtype: PredictionsSeriesGetResponse

        :Example:

        >>> from optimeering_beta import Configuration, OptimeeringClient
        >>> configuration = Configuration(host="https://beta.optimeering.com")
        >>> client = OptimeeringClient(configuration=configuration)
        >>> # Get filtered data point - replace ... with appropriate filters documented above
        >>> response = client.predictions_api.get_prediction_series(...)

        """  # noqa: E501

        _param = self._get_prediction_series_serialize(
            id=id,
            product=product,
            unit_type=unit_type,
            statistic=statistic,
            area=area,
            limit=None,
            offset=None,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PredictionsSeriesGetResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        paginated_response = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data
        if type(paginated_response) != dict:
            paginated_response._client = self.api_client
        next_page = paginated_response.next_page

        if next_page is None:
            return paginated_response

        # resolve extendable attribute
        attribute_to_extend_selected: Optional[str] = None
        if "series_id" in set(paginated_response.items[0].model_fields):
            extendable_attribute = {"datapoints", "events"}
            attribute_to_extend = extendable_attribute.intersection(set(paginated_response.items[0].model_fields))
            if len(attribute_to_extend) == 1:
                attribute_to_extend_selected = list(attribute_to_extend)[0]
            else:
                raise AttributeError("Failed to resolve extendable attribute.")

        api_key_configured = self.api_client.configuration.api_key is not None
        if api_key_configured:
            auth_header = {"apikey": self.api_client.configuration.api_key}
        while next_page is not None:
            if not api_key_configured:
                auth_header = {"Authorization": f"Bearer {self.api_client.token}"}

            next_pagination = self.api_client.call_api(
                method=_param[0],
                url=next_page,
                header_params={**_param[2], **auth_header},
                body=_param[3],
                post_params=_param[4],
                _request_timeout=_request_timeout,
            )
            next_pagination.read()
            next_pagination = self.api_client.response_deserialize(
                response_data=next_pagination,
                response_types_map=_response_types_map,
            ).data
            next_page = next_pagination.next_page

            if attribute_to_extend_selected:
                if next_pagination.items[0].series_id == paginated_response.items[-1].series_id:
                    # if series id is same, extract the first item and extend to last item
                    continued_data: List = getattr(next_pagination.items.pop(0), attribute_to_extend_selected)
                    previous_data: List = getattr(paginated_response.items[-1], attribute_to_extend_selected)
                    previous_data.extend(continued_data)
            paginated_response.items.extend(next_pagination.items)
        return paginated_response

    def _get_prediction_series_serialize(
        self,
        id,
        product,
        unit_type,
        statistic,
        area,
        limit,
        offset,
    ) -> RequestSerialized:
        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if id is not None:
            id = ",".join(str(i) for i in id)
            _query_params.append(("id", id))

        if product is not None:
            product = ",".join(str(i) for i in product)
            _query_params.append(("product", product))

        if unit_type is not None:
            unit_type = ",".join(str(i) for i in unit_type)
            _query_params.append(("unit_type", unit_type))

        if statistic is not None:
            statistic = ",".join(str(i) for i in statistic)
            _query_params.append(("statistic", statistic))

        if area is not None:
            area = ",".join(str(i) for i in area)
            _query_params.append(("area", area))

        if limit is not None:
            _query_params.append(("limit", limit))

        if offset is not None:
            _query_params.append(("offset", offset))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        if self.api_client.configuration.api_key is None:
            # inject azure token
            _header_params["Authorization"] = f"Bearer {self.api_client.token}"
        else:
            _header_params["apikey"] = self.api_client.configuration.api_key

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/predictions/series/",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
        )

    @validate_call
    def get_predictions(
        self,
        series_id: Annotated[
            Optional[List[StrictInt]],
            Field(description="Series ID to filter. If not specified, will return all series."),
        ] = None,
        start: Annotated[
            Optional[Any],
            Field(
                description="The first datetime to fetch (inclusive). Defaults to `1970-01-01 00:00:00+0000`. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)"
            ),
        ] = None,
        end: Annotated[
            Optional[Any],
            Field(
                description="The last datetime to fetch (exclusive). Defaults to '2999-12-30 00:00:00+0000'. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]],
        ] = None,
    ) -> PredictionsDataGetResponse:
        """Get Predictions

        Returns predictions

        :param series_id: Series ID to filter. If not specified, will return all series.
        :type series_id: List[StrictInt]
        :param start: The first datetime to fetch (inclusive). Defaults to `1970-01-01 00:00:00+0000`. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)
        :type start: Start
        :param end: The last datetime to fetch (exclusive). Defaults to '2999-12-30 00:00:00+0000'. Should be specified in ISO 8601 datetime or duration format (eg - '2024-05-15T06:00:00+00:00', 'P1H', '-P1W1D', etc.)
        :type end: End
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :return: Returns the result object.
        :rtype: PredictionsDataGetResponse

        :Example:

        >>> from optimeering_beta import Configuration, OptimeeringClient
        >>> configuration = Configuration(host="https://beta.optimeering.com")
        >>> client = OptimeeringClient(configuration=configuration)
        >>> # Get filtered data point - replace ... with appropriate filters documented above
        >>> response = client.predictions_api.get_predictions(...)

        """  # noqa: E501

        _param = self._get_predictions_serialize(
            series_id=series_id,
            start=start,
            end=end,
            limit=None,
            offset=None,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PredictionsDataGetResponse",
            "422": "HTTPValidationError",
        }
        response_data = self.api_client.call_api(*_param, _request_timeout=_request_timeout)
        response_data.read()
        paginated_response = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data
        if type(paginated_response) != dict:
            paginated_response._client = self.api_client
        next_page = paginated_response.next_page

        if next_page is None:
            return paginated_response

        # resolve extendable attribute
        attribute_to_extend_selected: Optional[str] = None
        if "series_id" in set(paginated_response.items[0].model_fields):
            extendable_attribute = {"datapoints", "events"}
            attribute_to_extend = extendable_attribute.intersection(set(paginated_response.items[0].model_fields))
            if len(attribute_to_extend) == 1:
                attribute_to_extend_selected = list(attribute_to_extend)[0]
            else:
                raise AttributeError("Failed to resolve extendable attribute.")

        api_key_configured = self.api_client.configuration.api_key is not None
        if api_key_configured:
            auth_header = {"apikey": self.api_client.configuration.api_key}
        while next_page is not None:
            if not api_key_configured:
                auth_header = {"Authorization": f"Bearer {self.api_client.token}"}

            next_pagination = self.api_client.call_api(
                method=_param[0],
                url=next_page,
                header_params={**_param[2], **auth_header},
                body=_param[3],
                post_params=_param[4],
                _request_timeout=_request_timeout,
            )
            next_pagination.read()
            next_pagination = self.api_client.response_deserialize(
                response_data=next_pagination,
                response_types_map=_response_types_map,
            ).data
            next_page = next_pagination.next_page

            if attribute_to_extend_selected:
                if next_pagination.items[0].series_id == paginated_response.items[-1].series_id:
                    # if series id is same, extract the first item and extend to last item
                    continued_data: List = getattr(next_pagination.items.pop(0), attribute_to_extend_selected)
                    previous_data: List = getattr(paginated_response.items[-1], attribute_to_extend_selected)
                    previous_data.extend(continued_data)
            paginated_response.items.extend(next_pagination.items)
        return paginated_response

    def _get_predictions_serialize(
        self,
        series_id,
        start,
        end,
        limit,
        offset,
    ) -> RequestSerialized:
        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if series_id is not None:
            series_id = ",".join(str(i) for i in series_id)
            _query_params.append(("series_id", series_id))

        if start is not None:
            _query_params.append(("start", start))

        if end is not None:
            _query_params.append(("end", end))

        if limit is not None:
            _query_params.append(("limit", limit))

        if offset is not None:
            _query_params.append(("offset", offset))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        if self.api_client.configuration.api_key is None:
            # inject azure token
            _header_params["Authorization"] = f"Bearer {self.api_client.token}"
        else:
            _header_params["apikey"] = self.api_client.configuration.api_key

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/api/predictions/",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
        )




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS

import errno
import os
import setuptools.build_meta

backend = setuptools.build_meta.__legacy__

dist_dir = "dist/optimeering-beta"
build_wheel = True
build_sdist = True
wheel_config_settings = {
    '--global-option': [
        '--python-tag',
        'py39.py310.py311',
    ],
}
sdist_config_settings = {
}

# Python 2.7 doesn't have the exist_ok arg on os.makedirs().
try:
    os.makedirs(dist_dir)
except OSError as e:
    if e.errno != errno.EEXIST:
        raise

wheel_path = backend.build_wheel(dist_dir, wheel_config_settings) if build_wheel else None
sdist_path = backend.build_sdist(dist_dir, sdist_config_settings) if build_sdist else None

if wheel_path:
    print("wheel: {wheel_path}".format(wheel_path=wheel_path))
if sdist_path:
    print("sdist: {sdist_path}".format(sdist_path=sdist_path))



# coding: utf-8

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Self


class PredictionsCreatedSeries(BaseModel):
    """
    PredictionsCreatedSeries
    """  # noqa: E501

    area: StrictStr = Field(description="Areas to be filtered. E.g. NO1, NO2")
    created_at: datetime
    description: Optional[StrictStr] = None
    id: StrictInt
    latest_event_time: Optional[datetime] = None
    product: StrictStr = Field(description="Product name for the series")
    statistic: StrictStr = Field(description="Type of statistics.")
    unit: StrictStr = Field(description="The unit for the series.")
    unit_type: StrictStr = Field(description="Unit type for the series")
    __properties: ClassVar[List[str]] = [
        "area",
        "created_at",
        "description",
        "id",
        "latest_event_time",
        "product",
        "statistic",
        "unit",
        "unit_type",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PredictionsCreatedSeries from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if latest_event_time (nullable) is None
        # and model_fields_set contains the field
        if self.latest_event_time is None and "latest_event_time" in self.model_fields_set:
            _dict["latest_event_time"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PredictionsCreatedSeries from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "area": obj.get("area"),
                "created_at": obj.get("created_at"),
                "description": obj.get("description"),
                "id": obj.get("id"),
                "latest_event_time": obj.get("latest_event_time"),
                "product": obj.get("product"),
                "statistic": obj.get("statistic"),
                "unit": obj.get("unit"),
                "unit_type": obj.get("unit_type"),
            }
        )
        return _obj



# coding: utf-8

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Dict, Optional, Set, Union

from optimeering_beta.models.prediction_float_model import PredictionFloatModel
from optimeering_beta.models.predictions_dict_model import PredictionsDictModel
from pydantic import BaseModel, ValidationError, field_validator
from typing_extensions import Self

PREDICTIONSINNER_ANY_OF_SCHEMAS = ["PredictionFloatModel", "PredictionsDictModel"]


class PredictionsInner(BaseModel):
    """
    PredictionsInner
    """

    # data type: PredictionsDictModel
    anyof_schema_1_validator: Optional[PredictionsDictModel] = None
    # data type: PredictionFloatModel
    anyof_schema_2_validator: Optional[PredictionFloatModel] = None
    actual_instance: Any = None
    any_of_schemas: Set[str] = {"PredictionFloatModel", "PredictionsDictModel"}

    model_config = {
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(actual_instance=kwargs)

    @field_validator("actual_instance")
    def actual_instance_must_validate_anyof(cls, v):
        instance = PredictionsInner.model_construct()  # noqa: F841
        error_messages = []
        # validate data type: PredictionsDictModel
        try:
            return PredictionsDictModel.validate(v)
        except ValidationError:
            if not isinstance(v, PredictionsDictModel):
                error_messages.append(f"Error! Input type `{type(v)}` is not `PredictionsDictModel`")
            else:
                return v

        # validate data type: PredictionFloatModel
        try:
            return PredictionFloatModel.validate(v)
        except ValidationError:
            if not isinstance(v, PredictionFloatModel):
                error_messages.append(f"Error! Input type `{type(v)}` is not `PredictionFloatModel`")
            else:
                return v

        if error_messages:
            # no match
            raise ValueError(
                "No match found when setting the actual_instance in PredictionsInner with anyOf schemas: PredictionFloatModel, PredictionsDictModel. Details: "
                + ", ".join(error_messages)
            )
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        # anyof_schema_1_validator: Optional[PredictionsDictModel] = None
        try:
            instance.actual_instance = PredictionsDictModel.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # anyof_schema_2_validator: Optional[PredictionFloatModel] = None
        try:
            instance.actual_instance = PredictionFloatModel.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if error_messages:
            # no match
            raise ValueError(
                "No match found when deserializing the JSON string into PredictionsInner with anyOf schemas: PredictionFloatModel, PredictionsDictModel. Details: "
                + ", ".join(error_messages)
            )
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], PredictionFloatModel, PredictionsDictModel]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())



# coding: utf-8

"""
    Optimeering

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import inspect
import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, Iterable, List, Optional, Set

import optimeering_beta
from optimeering_beta.models.predictions_created_series import PredictionsCreatedSeries
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing_extensions import Self


class PredictionsSeriesGetResponse(BaseModel):
    """
    PredictionsSeriesGetResponse
    """  # noqa: E501

    items: List[PredictionsCreatedSeries]
    next_page: Optional[StrictStr] = Field(default=None, description="The next page of results (if available).")
    _client: Any = None
    __properties: ClassVar[List[str]] = ["items", "next_page"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PredictionsSeriesGetResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in items (list)
        _items = []
        if self.items:
            for _item in self.items:
                if _item:
                    _items.append(_item.to_dict())
            _dict["items"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PredictionsSeriesGetResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "items": [PredictionsCreatedSeries.from_dict(_item) for _item in obj["items"]]
                if obj.get("items") is not None
                else None,
                "next_page": obj.get("next_page"),
            }
        )
        return _obj

    @property
    def series_ids(self) -> List[int]:
        """Returns all the series ids included in the response"""
        iterate_over: Iterable
        if "items" in self.model_fields:
            iterate_over = self.items
        else:
            iterate_over = self
        return list({getattr(i, "id") for i in iterate_over})

    def datapoints(
        self,
        start: Optional[datetime | StrictStr] = None,
        end: Optional[datetime | StrictStr] = None,
        include_history: Optional[StrictBool] = None,
    ) -> optimeering_beta.models.PredictionsDataGetResponse:
        """
        Returns data points associated with

        :param start: The first datetime to fetch (inclusive). Defaults to current time. Should be specified in ISO 8601 format (eg - '2024-05-15T06:00:00+00:00'). Also supports delta formats (e.g. H+1,D-1,W-1)
        :param end: The last datetime to fetch (exclusive). Defaults to 2099-12-30. Should be specified in ISO 8601 format (eg - '2024-05-15T08:00:00+00:00'). Also supports delta formats (e.g. H+1,D-1,W-1)
        :param include_history: Include historical data into the response. Defaults to False. This argument has no effect if the underlying API does not support getting history.
        """
        if self._client is None:
            raise AttributeError("Cannot call datapoints method on this instance. The client has not been setup.")
        method_for_operation = optimeering_beta.PredictionsApi(api_client=self._client).get_predictions

        extra_params: Dict[str, Any] = {}
        valid_arguments = list(inspect.signature(method_for_operation).parameters.keys())
        if "include_history" in valid_arguments:
            extra_params["include_history"] = include_history

        return method_for_operation(series_id=self.series_ids, start=start, end=end, **extra_params)

    def __iter__(self):
        """Iteration method for generated models"""
        if isinstance(self, list):
            return (i for i in self)
        elif "items" in self.model_fields:
            return iter(self.items)
        else:
            raise AttributeError("This object does not support iteration.")
